%% 1 Calibrate the Economy

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calibrate the economy at an monthly frequency
% 
%  Xiaoji Lin and Lu Zhang, Inc. 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% Notation: k denotes capital, x is TFP shock, z is idiosyncratic productivity

clear all; format short; format compact; clc;

% pricing kernel: log(M_{t,t+1}) = log(beta) + gamma_t*(x_t -x_t+1) 
% where gammat = gam0 + gam1*a_t;

beta    =  0.9999;					                    % discount Factor in static case 
gam0    =  17;                                          % constant price of risk
gam1    =  -1000;                                        % time-varying price of risk

% technology parameter values
alpha   = 0.7;                                          % Capital's Share in Ouput
delta   = 0.01;                                         % Depreciation Rate of Capital

% parameters in adjustment costs specification
% assume asymmetric convex and nonconvex adjustmetn costs: G(i,k) = {a0*k + b0*i + .5*c0*(i/k)^(t)*k}*1{I>=0) + {a1*k + b1*i + 0.5*c1*(i/k)^(t)*k}*1{I<0)
a0 = 0.01;                                                % upward nonconvex costs  
a1 = 0.03;                                                % downward nonconvex costs
b0 = 1;                                                   
b1 = 1;
c0  = 20;                                                 % upward convex costs  
c1  = c0*10;                                              % downward convex costs
t  = 2;                                                   % curvature of adjustment costs

% some stead state relations
kstar      = 1;
istar      = delta*kstar;
irate_star = istar/kstar;
qstar      = 1 + c0*irate_star;

% aggregate shocks : x
nx      = 15   
rhox    = 0.95^(1/3);                                       % calibrated to match rhox = 0.95 quarterly
stdx    = 0.007/sqrt(1+rhox^2+rhox^4);                      % calibrated to match stdx = .007 quarterly
xbar    = -3.65;                                            % calibrated to make average k around 
xdev    = 2*stdx/sqrt((1 - rhox^2)*(nx - 1));  
[Qx, x] = rouwTrans(rhox, xbar, xdev, nx);                  %  Rouwenhorst (1995)
xmin    = min(x);
xmax    = max(x);
imx     = ceil(nx/2);

% idiosyncratic shocks -- Rouwenhorst (1995) method
nz      = 9
zbar    = 0;
rhoz    = 0.97;                                     
stdz    = 0.10;                                         % benchmark level is .10
zdev    = 2*stdz/sqrt((1 - rhoz^2)*(nz - 1));           % do ''help rouwTrans'' to understand the formular
[Qz, z] = rouwTrans(rhoz, zbar, zdev, nz);
zmin    = min(z);
zmax    = max(z);
imz     = ceil(nz/2);                                   % index of the average shock
uQz     = Qz^1000;                                      % unconditional distribution of z
uQz     = uQz(:, 1);

% Fixed Costs
f  = 0.0032;                                             % calibrated to match market to book ratio

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Real interest rate solved on the grid of x
rf  = getRfcc(x, Qx, beta, gam0, gam1)';
if 1
% aggregate shocks on discrete state space
[sx]  = simuX(2050000,imx,Qx,x);
sx(1 : 100000) = [];
% real interest rate
rfs = interp1(x, rf, sx, 'linear', 'extrap');
mrf  = (mean(rfs)-1)*12
vrf  = std(rfs)*sqrt(12)
clear sx rfs mrf vrf
end

% average Sharpe ratio
Sharpe = sqrt(12)*sqrt(exp(gam0^2*stdx^2) * (exp(gam0^2*stdx^2) - 1)) /exp(1/2*gam0^2*stdx^2)   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set up the grid for k
kmin  = 0.01;
kmax  = 2.5*kstar;

if 0
    % construct grid for physical capital stock --- use equal-spaced grid
    nk = 200;
    kgrid  = linspace(kmin, kmax, nk)';
else
    % construct grid for capital stock --- use nonequal-spaced grid via McGrattan method
    next  = 1;
    ik    = 1;
    kgrid     = kmin;
    % recursive construction: some experiment shows that 50 grid points are enough
    while next < kmax
        ik    = ik + 1; 
         next = kgrid(ik - 1) + .0025*exp(0.05205*(ik - 2));                    %  100 Grid Points (kmax = 3)
        if next < kmax, kgrid(ik) = next; end
    end
    kgrid       = kgrid';
end
clear next ik 
kmax  = max(kgrid)
nk    = length(kgrid)
imk   = find(kgrid==kstar);

% construct grid for kprime
kgridp = kgrid;
nkp    = length(kgridp)

save para
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2. Value Function Iterations

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computes the value of the firm with one capital and two stochastic shocks
% aggregate shock and idiosyncratic shock
%
%  Xiaoji Lin and Lu Zhang, Inc. 2012.
% ----------------------------------------------------
% pricing kernel: log(M_{t,t+1}) = log(eta) + gamma_t*(x_t -x_t+1) 
% where gammat = gam0 + gam1*x_t;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; clc
load para.mat

% Initialize relevant variables
nkm     =  200;                                     % No. of grid points for investment
dev     = .01*delta*kmax;                          % deviation used in investment grid

V0      = kgrid.^alpha./(1-beta);
I       = zeros(nkp,nk,nx,nz);
G       = zeros(nkp,nk,nx,nz);
Div     = zeros(nkp,nk,nx,nz);
Inew    = zeros(nkm,1);
Gnew    = zeros(nkm,1);
knew    = zeros(nkm,1);

V       = zeros(nk,nx,nz);
Y       = zeros(nk,nx,nz);
New_V   = zeros(nk,nx,nz);
V_ss    = zeros(nkp*nx,nz);    
EVP     = zeros(nkp,nx,nz);                                % Initialize Expected EV
EVP_New = zeros(nkm,1);                                % Initialize Expected EV

Ipnew   = zeros(nk,nx,nz);
Ipold   = zeros(nk,nx,nz);
indexold   = zeros(nk,nx,nz);
index      = zeros(nk,nx,nz);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Each block is for one state of idiosyncratic nature, i.e., each state of idiosyncratic productivity has a 3-dimentional array of V(k,x,z);
for i = 1 : nx
    for j = 1 : nz
        V(:,i,j) = exp(x(i)+z(j))* V0;
    end
end
                
for h = 1: nz
    for j = 1 : nx
        Y(:,j,h) = exp((x(j)+z(h)))*(kgrid.^alpha);                     % output
        for i = 1 : nk       
             I(:,i,j,h)  = kgridp - (1 - delta)*kgrid(i);               % investment
             G(:,i,j,h)  = (a0*kgrid(i) + b0*I(:,i,j,h) + 0.5*c0.*(I(:,i,j,h)./kgrid(i)).^t*kgrid(i)).*(I(:,i,j,h)>0)...
                           +(a1*kgrid(i) +  b1*I(:,i,j,h) + 0.5*c1.*(I(:,i,j,h)./kgrid(i)).^t*kgrid(i)).*(I(:,i,j,h)<0) ; 
             Div(:,i,j,h) = Y(i,j,h) - f- G(:,i,j,h);                   % dividend
         end
    end
end

clear i j;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Pricing kernel 
difx = zeros(nx,nx); detM = zeros(nx,nx);
gama = gam0 + gam1*(x - xbar);
for i = 1 : nx
      difx(:,i) = x(i) - x;
      detM(:,i) = gama(i)*difx(:,i);
end 
stoM =  beta*exp(detM);
Qxx  = stoM.*Qx;                                        % effectively "transition" matrix

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Iterate the Bellman Equation
iter = 0;                                               % Begin iterations
tor  = 1D-3*(1-beta);                                   % tolerance
iterMax = 2200;                                         % maximum No of iterations
cutoff1 = 600;                                          % cutoff point to start interpolate investment on a fine grid
cutoff2 = 900;                                          % cutoff point to start interpolate investment on a finer grid   
errV    = zeros(iterMax,1);                             % iteration error for value function
errI    = zeros(iterMax,1);                             % iteration error for policy function

tic;                                                    % start stoptime watcher
while iter < iterMax
    
        iter    = iter + 1;        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coarse grid
        % Compute expected firm value along x dimension
       for i = 1:nz
           EVP(:,:,i) = V(:,:,i)*Qxx;
       end

       % Compute expected firm value along z dimension
       % transform V(nk_m,nk_u,nx,nz) to V(nk_m*nk_u,nx,nz)  
       V_ss  = reshape(EVP, nkp*nx, nz)*Qz;             
       EVP   = reshape(V_ss, nkp, nx, nz);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Value maximization

      if iter <cutoff1  
          for h = 1: nz
                for j = 1 : nx
                    for i = 1 : nk                  
                         % Bellman equation: V(k,x,z) = Y-f-I-G + E{M*V(k',x',z')}
                        [New_V(i,j,h) indexold(i,j,h)] = max( Div(:,i,j,h) + EVP(:,j,h) );
                        Ipold(i,j,h)       = I(indexold(i,j,h),i,j,h) ;
                    end
               end
          end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Update investment grid
          elseif (iter >= cutoff1)&& (iter<cutoff2)

                  for h = 1: nz
                        for j = 1 : nx
                            for i = 1 : nk                  
 
                                    Inew            = linspace( Ipold(i,j,h)-dev, Ipold(i,j,h)+dev,nkm)';   % fine investment grid
                                    knew            = min(kmax, max(Inew + (1 - delta)*kgrid(i),kmin));     % fine capital grid
                                    EVP_New         = interp1(kgridp, EVP(:,j,h), knew, 'linear');
                                    Gnew            = (a0*kgrid(i) + b0*Inew + 0.5*c0.*(Inew./kgrid(i)).^t*kgrid(i)).*(Inew>0)...
                                                        +(a1*kgrid(i) + b1*Inew + 0.5*c1.*(Inew./kgrid(i)).^t*kgrid(i)).*(Inew<0) ; 
                                    [New_V(i,j,h) index(i,j,h)] = max( Y(i,j,h) - f - Gnew + EVP_New);
                                    Ipnew(i,j,h)    = Inew(index(i,j,h));
                             end
                         end
                  end
           
                  errI(iter) =  abs(max((max(max(Ipnew - Ipold)))));
                  Ipold = Ipnew;

 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Update finer investment grid

        elseif iter >= cutoff2 
 
                  for h = 1: nz
                        for j = 1 : nx
                            for i = 1 : nk                  
 
                                    Inew            = linspace( Ipold(i,j,h)-dev, Ipold(i,j,h)+dev,nkm)';   % fine investment grid
                                    knew            = min(kmax, max(Inew + (1 - delta)*kgrid(i),kmin));     % fine capital grid
                                    EVP_New         = interp1(kgridp, EVP(:,j,h), knew, 'cubic');
                                    Gnew            = (a0*kgrid(i) + b0*Inew + 0.5*c0.*(Inew./kgrid(i)).^t*kgrid(i)).*(Inew>0)...
                                                        +(a1*kgrid(i) + b1*Inew + 0.5*c1.*(Inew./kgrid(i)).^t*kgrid(i)).*(Inew<0) ; 
                                    [New_V(i,j,h) index(i,j,h)] = max( Y(i,j,h) - f - Gnew + EVP_New);
                                    Ipnew(i,j,h)    = Inew(index(i,j,h));
                             end
                         end
                  end

                  errI(iter) =  abs(max((max(max(Ipnew - Ipold)))));
                  Ipold = Ipnew;
           
      end
      
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Check Convergence
  
        errV(iter) =  abs(max((max(max(V - New_V)))));
        if errV(iter) < tor && errI(iter)<tor
            break
        end

        %Update
        V     = New_V;
        % Visual
        fprintf('errV = %6.4d errI = %6.4d   iteration = %6.4f   elapsed time: %6.4f\n', [errV(iter) errI(iter) iter toc]);
end 
  
I = Ipnew;

clear V0 G Y Div New_V V_s EV V_ss Qxx Inew Gnew knew EVP EVP_New kp indexold index Ipold Ipnew

% Save all variables
save VF2

%% 3. Policy Functions 

clear all; clc
load VF2.mat

Ikrate = zeros(nk,nx,nz);
kprime = zeros(nk,nx,nz);
Y      = zeros(nk,nx,nz);
Z      = zeros(nk,nx,nz);
X      = zeros(nk,nx,nz);
G      = zeros(nk,nx,nz);
Div    = zeros(nk,nx,nz);
P      = zeros(nk,nx,nz);
RF     = zeros(nk,nx,nz);
EV     = zeros(nk,nx,nz);
ExpRet = zeros(nk,nx,nz);
ExRet  = zeros(nk,nx,nz);
Beta   = zeros(nk,nx,nz);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get capital,investment,output, divident, adj. costs, and ex. dividend stock price
for h = 1 : nz
        for j = 1 : nx
           
            kprime(:,j,h) = I(:,j,h) + (1 - delta)*kgrid;                   % optimal capital
            Ikrate(:,j,h) = I(:,j,h)./kgrid;                                % investment rate
            Y(:,j,h)      = exp((x(j)+z(h)))*(kgrid.^alpha);                % output
            G(:,j,h)      = (a0*kgrid + b0*I(:,j,h) + 0.5*c0.*(I(:,j,h)./kgrid).^t.*kgrid).*(I(:,j,h)>0)...
                            + (a1*kgrid + b1*I(:,j,h) + 0.5*c1.*(I(:,j,h)./kgrid).^t.*kgrid).*(I(:,j,h)<0) ;
            Div(:,j,h)    = Y(:,j,h) - f - G(:,j,h);                        % dividend
            P(:,j,h)      = V(:,j,h) -Div(:,j,h);                           % ex. dividend firm value
        end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% to get V_prime
% Construct 3-D grid using 'meshgrid'
[xs,ks,zs] = meshgrid(x', kgrid, z);

for i = 1:nz
       EV(:,:,i) = V(:,:,i)*Qx;
end

% compute expected value 
V_ss  = reshape(EV,nk*nx,nz)*Qz;             
EV    = reshape(V_ss, nk, nx, nz);

X = repmat(x',[nk,1,nz]);
for i = 1:nz
    Z(:,:,i) = repmat(z(i),[nk,nx,1]);
end

EVprime  = interp3(xs,ks,zs,EV,X,kprime,Z,'cubic');
Vprime  = interp3(xs,ks,zs,V,X,kprime,Z,'cubic');

% calculate Conditional Expected Returns following Zhang (2005): Et{r(t+1)}=Et{V(t+1)}/[V(t)-Div(t)]
ExpRet    = EVprime./P;
RetPrime  = Vprime./P;

% beta{t}= - COVt[R{t+1}, M{t+1}]/VARt[M{t+1}]; 
% Rf{t} = 1/Et{M(t+1)}: Construct EM
EM     =  getEM(x,Qx,beta,gam0, gam1);
RF1    = 1./EM;

for j = 1 : nx
     RF(:,j,:) = repmat(RF1(j), [nk,1,nz]);
end

% Excess returns;
ExRet = ExpRet - RF;

% calculate Lambda: Lambda{t} = VARt[M{t+1}]/Et[M{t+1}]
Lambda = getLambda(x, beta, gam0, gam1, xbar, rhox, stdx);
% get conditional  beta{t} = [ Et{r(t+1)} - Rf{t}] /lamda{t}
for i = 1 : nx
    Beta(:,i,:) = ExRet(:,i,:)/Lambda(i);
end

clear RF EM EVprime X Z EV V_ss xs ks zs Lambda RF
save Policy

%% 4. Simulate a panel firms and use the last period of capital and idiosyncratic shock as initial values for main simulations

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% simu:
% simulates a stationary distribution of N firms and T time periods using interpolation
% 
%  Xiaoji Lin and Lu Zhang 2012.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc

load Policy;                                        % policy Function

% specify number of firms and time periods to simulate
N = 5000                                             % No of firms
T = 500                                              % No of time periods 
xinl = xbar;

% Initializing the simulation
ksim     = ones(N, T)*kstar;
Vsim     = zeros(N,T);
Psim     = zeros(N,T);
xid      = zeros(T, 1);                                  % aggregate shock
zid      = zeros(N, T);                                  % idiosyncratic shock
sysrand  = rand(T, 1);                                   
idiorand = rand(N, T);                               
xid(1)   = imx;                                          % initialize x
zid(:, 1) = ones(N, 1)*imz;                              % initialize z

cumQx = cumsum(Qx);
cumQz = cumsum(Qz);

simx    = zeros(T,1);
zsim    = ones(N,T)*zbar;

% Construct 2-D grid using 'meshgrid'
[zs,ks] = meshgrid(z,kgrid);

tic;
for j = 2:T
    
         xid(j) = find(cumQx(:, xid(j-1)) >= sysrand(j), 1); 
         simx(j-1) = x(xid(j));

         for n = 1:N
             zid(n, j)   = find(cumQz(:, zid(n, j-1)) >= idiorand(n, j), 1); 
             zsim(n,j-1) = z(zid(n, j));
         end
        % update next period capital
        ksim(:,j)     = interp2(zs, ks , squeeze(kprime(:,xid(j),:)),   zsim(:,j-1),ksim(:,j-1),'linear');       % K
        Vsim(:,j)     = interp2(zs, ks , squeeze(V(:,xid(j),:)),        zsim(:,j-1),ksim(:,j-1),'linear');       % V
        Psim(:,j)     = interp2(zs, ks , squeeze(P(:,xid(j),:)),        zsim(:,j-1),ksim(:,j-1),'linear');       % P
        
        ksim(:,j)    = min(kmax,max(ksim(:,j),kmin));                                                            % boundary control
end; 

fprintf('simulation periods = %6.4f   elapsed time: %6.4f\n', [j toc]) 

kd0 = ksim(:, end);
zd0  =zsim(:, end);

clear N T xid zid sysrand iodrand cumQx cumQx xid zid zs ks;
% save simulation results
save simu;
 
%% 5. Main Simulations
% simulate panels of data and implement all the analysis

clear all; clc;

%load simulated data
load simu

clear ksim simx zsim N T P Y K Q I Ikrate Mkratio Ret Y j;

% specify number of sample and time periods in each sample to simulate
nsim = 1000                                                % No of samples
N  = 5000                                               % No of firms
T  = 1000                                               % No of time periods
cutoff = T-607;                                         % get rid of initial 1:cutoff periods of data
L      = T-cutoff-1-18;

% preallocation
ik_mean     = zeros(nsim,1);                ik_ts_std = zeros(nsim,1);          ik_xs_std=zeros(nsim,1);            
mb_mean     = zeros(nsim,1);                mb_ts_std = zeros(nsim,1);          mb_xs_std=zeros(nsim,1);            
rfs         = zeros(T-cutoff-1,nsim);       rfsstd    = zeros(nsim,1);          mktRet     = zeros(T-cutoff-1,nsim);       
Exmkt       = zeros(T-cutoff-1,nsim);       Exmkt_std  = zeros(nsim,1);         mktRet_std = zeros(nsim,1);
corr_beta_bm= zeros(nsim,1);                R2         = zeros(nsim,1);         B1 = zeros(nsim,1);

tic;
for (s = 1 : nsim)                                       

     % simulate a panel of N firms each with T-cutoff periods
            [Ret,VF, Divd, Ps,K,Is,BM,Y,BetaS,ExpRetS,simx]  = simuSSS(kd0,zd0,N, T, cutoff);
            
    % simulated SDF
            Msim   = beta*exp((gam0+gam1*(simx(1:end-1)-xbar)).*(simx(1:end-1)-simx(2:end)));
            M(:,s) = Msim(cutoff:end-1);
    % aggregate D/P
            Div_a = sum(Divd); P_a = sum(Ps);
            DP    = Div_a./P_a;                
            N2    = size(Ret,1);
     % summary statistics
            mb_mean(s)    = mean(mean(1./BM,2));                           % average of market to book ratio
            mb_ts_std(s)  = mean(std(1./BM'));                             % std dev of market to book ratio
            [I_A Ikrate]  = monthlyToAnnualIK(Is,K);                       % Aggregation over time: timing is that the starting time is Jan.
            ik_mean(s)    = mean(mean(Ikrate,2));                          % average of I/K
            ik_ts_std(s)  = mean(std(Ikrate'));                            % time series std dev of I/K

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% correlations and risk-adjusted BM
            [NN TT] = size(BetaS); rsd_bm =[];
            for i=1:TT
                corr_beta_bm_temp(i)    = corr(BetaS(:,i),BM(:,i));
                slp                     = [ones(NN,1) BetaS(:,i)]\BM(:,i);  % regress BM on beta to get risk-adjusted BM
                rsd_bm(:,i)             = BM(:,i) - [BetaS(:,i)]*slp(2);    % risk-adjusted BM
            end
            corr_beta_bm(s) = mean(corr_beta_bm_temp);
            
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% real interest rate solved on the grid of x
            rf                  = getRfcc(x, Qx, beta, gam0, gam1)';
            rfss                = interp1(x, rf, simx, 'linear', 'extrap');   
            rfs(:,s)            = rfss(cutoff+1:end-1);                     % risk free rate
            rfsstd(s)           = std(rfs(:,s));                            % volatility of risk free rate
            [mktRet(:,s)]       = mktRetVW(Ps, Ret);                        % market returns
            [mktRet_std(s)]     = std(mktRet(:,s));                         % volatility of market returns
            [Exmkt(:,s)]        = mktRet(:,s)-rfs(:,s);
            [Exmkt_std(s)]      = std(Exmkt(:,s));
            clear rf rfss;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adjust timing to be consistent with Fama-French 1992/1993       

            rfss       = rfs(12+6:end,s);                                 % Align timing with FF3 factors and other VW portfolio returns                               
            mkt        = mktRet(12+6:end,s);                              % Align timing with FF3 factors and other VW portfolio returns
            msdf       = M(12+6:end,s);
            RetNew     = Ret(:,12+6:end);                                 % Align timing with FF3 factors and other VW portfolio returns
            BMs        = BM(:,12:end);                                    % Align timing with FF3 factors and other VW portfolio returns  
            Pss        = Ps(:,12+6:end);                                  % Align timing with FF3 factors and other VW portfolio returns
            Betass     = BetaS(:,12+6:end);
            rsd_bmss   = rsd_bm(:,12:end);
            dp         = DP(12+6:end);
            
            % Form Fama-French 3 factors 2 by 3 size/bm portfolios to construct FF
            [FF6_VW(:,:,:)]  =  portSortVW6(Pss, BMs, RetNew);          

           % to get Fama and French 3 factor and CAPM rolling loadings 
            RolT  = 36;                                            % Length of rolling window
            RolN  = T-cutoff-1-RolT-12-6+1;                        % No. of rolling regressions
            L2    = floor((L-RolT+1-18)/12)*12;                    % effective length

            for h = 1 : RolN
                FF3LD(:,:,h) =  FF3SLP(rfss(h:RolT+h-1), mkt(h:RolT+h-1),  FF6_VW(:,:,h:RolT+h-1), RetNew(:,h:RolT+h-1)); % Timing of rf, mkt, and Ret are all consistent:starting from July
                SDFLD(:,:,h) = -MLD(msdf(h:RolT+h-1),  RetNew(:,h:RolT+h-1));                            % SDF rolling regression
                Beta_TV(:,h) = -TVCAPM(dp(1:RolT+h-1)',msdf(1:RolT+h-1),RetNew(:,1:RolT+h-1));           % expanding window 
            end
              
            Beta_HML = reshape(FF3LD(:,4,:),[N2 RolN] ); clear FF3LD
            Beta_SDF = reshape(SDFLD(:,2,:),[N2 RolN]);  clear SDFLD
            
% sequential sorting: first BM then beta          
            % 1.0 HML loadings
            [BMH25(:,:,:,s)] = port5by5SESortVW(BMs(:,12+RolT:end),Beta_HML(:,6:end),...
                                                 Pss(:,18+RolT:end), RetNew(:,18+RolT:end)-ones(N2,1)*rfss(18+RolT:end)');
                                                  
             %1.1. Use Conditional beta
            [BMCBT25(:,:,:,s)] = port5by5SESortVW(BMs(:,12+RolT:end),Betass(:,12+RolT:end),...
                                                Pss(:,18+RolT:end), RetNew(:,18+RolT:end)-ones(N2,1)*rfss(18+RolT:end)');
            
             % 1.2 SDF rolling beta
            [BSDFH25(:,:,:,s)] = port5by5SESortVW(BMs(:,12+RolT:end),Beta_SDF(:,6:end),...
                                                 Pss(:,18+RolT:end), RetNew(:,18+RolT:end)-ones(N2,1)*rfss(18+RolT:end)');                                
                                
            % 1.3. Use time-varying beta conditional on DP
            [BMTVBT25(:,:,:,s)] = port5by5SESortVW(BMs(:,12+RolT:end),Beta_TV(:,6:end),...
                                                 Pss(:,18+RolT:end), RetNew(:,18+RolT:end)-ones(N2,1)*rfss(18+RolT:end)');
                                                                     
% risk-adjusted BM
% sequential sorting: first risk-adjusted BM then beta          
            % 2.0 HML loadings                              
            [BMH25rsd(:,:,:,s)] = port5by5SESortVW(rsd_bmss(:,12+RolT:end),Beta_HML(:,6:end),...
                                                 Pss(:,18+RolT:end), RetNew(:,18+RolT:end)-ones(N2,1)*rfss(18+RolT:end)');
             %2.1. Use conditional beta
            [BMCBT25rsd(:,:,:,s)] = port5by5SESortVW(rsd_bmss(:,12+RolT:end),Betass(:,12+RolT:end),...
                                                Pss(:,18+RolT:end), RetNew(:,18+RolT:end)-ones(N2,1)*rfss(18+RolT:end)');
                                
% sequential sorting: first beta then BM          
            % 3.0. HML loadings
            [HBM25(:,:,:,s)] = port5by5SESortVW(Beta_HML(:,6:end),BMs(:,12+RolT:end),...
                                                 Pss(:,18+RolT:end), RetNew(:,18+RolT:end)-ones(N2,1)*rfss(18+RolT:end)');
             % 3.1. Use conditional beta
            [CBTBM25(:,:,:,s)] = port5by5SESortVW(Betass(:,12+RolT:end),BMs(:,12+RolT:end),...
                                                Pss(:,18+RolT:end), RetNew(:,18+RolT:end)-ones(N2,1)*rfss(18+RolT:end)');                                           

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % 10 BM value-weighted returns 
             [FF10_VW(:,:)]    =  portSort10VW(BMs, Pss, RetNew-ones(N2,1)*rfss');             % 10 portfolios by book-to-market
             [FF11_VW(1:10,:,s)] = FF10_VW(:,:);                                               % 10 book-to-market portfolios and the value premium
             [FF11_VW(11,:,s)]   = FF10_VW(10,:) - FF10_VW(1,:);                               % value premium
             
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%             
    % CAPM          
            T_temp  = size(Exmkt(12+6:end,s),1);
            T_temp2 = floor(T_temp/12)*12;
            Exmkt_temp  = Exmkt(12+6:end,s);
            Exmkt_temp2 = Exmkt_temp(1:T_temp2);
            
            [bv sebv R2v R2vadj] = CAPM(Exmkt_temp2, FF11_VW(:,:,s));
            alpha_C(:,s)        = bv(1,:);                         beta_C(:,s)   = bv(2,:);                % intercept and slope of CAPM
            t_alpha_C(:,s)      = bv(1,:)./sebv(1,:);              t_beta_C(:,s) = bv(2,:)./sebv(2,:);     % t-stat for intercept and slope of CAPM
            R2v_C(:,s)          = R2v;                             R2vadj_C(:,s) = R2vadj;                 % Adj. R-square  
             
            fprintf('sample = %6.4f   elapsed time: %6.4f\n', [s toc])  
            clear Beta_TV
end 

%% Annualized sample averages

% market returns
Stock_mkt_ret_VW = mean(mean(mktRet-1))*1200;                     % value-weighted returns
Stock_mkt_VW_std = mean(mktRet_std)*sqrt(12)*100;                 % value weighted stock market return std. dev.

MKT_EX      = mean(mean(Exmkt))*1200;
MKT_EX_std  = mean(Exmkt_std)*sqrt(12)*100;
SP          = MKT_EX/MKT_EX_std;                                    % Sharpe ratio
RF          = mean(mean(rfs) -1)*1200;                              % Risk free rate
RF_std      = mean(rfsstd)*sqrt(12)*100;                            % Risk free rate volatility

% market to book
MB = mean(mb_mean);
% investment rate
IK = mean(ik_mean);
IK_ts_std = mean(ik_ts_std);

% corr(BM,beta)
Corr_BM_Beta = mean(corr_beta_bm);

% 10 book-to-market portfolios
BTM11_VW = mean(mean(FF11_VW,3),2)*100;
BTM11_VW_STD = mean(std(FF11_VW,0,2),3)*100;                        % std. dev

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Average excess returns of 25 portfolios
% HML beta
BMH_mean = mean(mean(BMH25,4),3)*100;
% SDF beta
BSDFH_mean = mean(mean(BSDFH25,4),3)*100;
% Conditional beta
BMCBT_mean = mean(mean(BMCBT25,4),3)*100;
% time varying beta conditional on PD
BMTVBT25_mean = mean(mean(BMTVBT25,4),3)*100; 

% Risk adjusted BM: Average excess returns of 25 portfolios
% HML beta
BMHrsd_mean = mean( mean(BMH25rsd,4),3)*100;
% Conditional beta
BMCBTrsd_mean = mean(mean(BMCBT25rsd,4),3)*100;

% Sequential: first beta then BM
HBM_mean = mean( mean(HBM25,4),3)*100;
% Conditional beta
CBTBM_mean = mean(mean(CBTBM25,4),3)*100;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% unconditional CAPM: 10 BM portfolios
alpha_CAPM     = mean(alpha_C,2)*100; beta_CAPM    = mean(beta_C,2);             
t_alpha_CAPM   = mean(t_alpha_C,2);   t_beta_CAPM  = mean(t_beta_C,2);       
R2vadj_CAPM    = mean(R2vadj_C,2);       

alpha_CAPM_95Conf = prctile(alpha_C',[2.5,97.5])*100;
beta_CAPM_95Conf  = prctile(beta_C',[2.5,97.5]);

save panIE;
%% Print Results of Main Moments
head = {'Summary Statistics';}; 
fmt = ' %G6.2 '; 
row = { 'Risk-Free Rate';'Vol RF'; 'Sharpe Ratio'; 'Mean IK'; 'Vol IK';'Market-to-Book';'Corr(BM,Beta)';};
Moments=[ RF; RF_std;SP;  IK; IK_ts_std; MB;Corr_BM_Beta;] ;
texprint(Moments, ' %6.2f ', head, row);

% Print 25 portfolio results
% HML Beta
clear Forecasts Port2
fprintf('25 BM and HML Beta Portfolios \n');
head = {'L HML';'2';'3';'4';'H HML';};
fmt = ' %G6.2 ';
row1 = { 'L BM'; '2';'3';'4';'H BM';'Average';};
Port2=zeros(6,5);
Port2(1:5,:)= BMH_mean;
Port2(6,:)  = mean(BMH_mean);
Forecasts =[Port2; ];
texprint(Forecasts, ' %12.3f ', head, row1);

% Conditional Beta 
clear Forecasts Port2
fprintf('25 BM and Conditional Beta Portfolios \n');
head = {'L Beta';'2';'3';'4';'H Beta';};
fmt = ' %G6.2 ';
row1 = { 'L BM'; '2';'3';'4';'H BM';'Average';};
Port2=zeros(6,5);
Port2(1:5,:)= BMCBT_mean;
Port2(6,:)  = mean(BMCBT_mean);
Forecasts =[Port2; ];
texprint(Forecasts, ' %12.3f ', head, row1);
 
% SDF Beta 
clear Forecasts Port2
fprintf('25 BM and SDF Beta Portfolios \n');
head = {'L Beta';'2';'3';'4';'H Beta';};
fmt = ' %G6.2 ';
row1 = { 'L BM'; '2';'3';'4';'H BM';'Average';};
Port2=zeros(6,5);
Port2(1:5,:)= BSDFH_mean;
Port2(6,:)  = mean(BSDFH_mean);
Forecasts =[Port2; ];
texprint(Forecasts, ' %12.3f ', head, row1);
 
% Time varying conditional on PD Beta 
clear Forecasts Port2
fprintf('25 BM and Time Varying Beta Conditional On DP Portfolios \n');
head = {'L Beta';'2';'3';'4';'H Beta';};
fmt = ' %G6.2 ';
row1 = { 'L BM'; '2';'3';'4';'H BM';'Average';};
Port2=zeros(6,5);
Port2(1:5,:)= BMTVBT25_mean;
Port2(6,:)  = mean(BMTVBT25_mean);
Forecasts =[Port2; ];
texprint(Forecasts, ' %12.3f ', head, row1);

% Risk adjusted BM
% HML Beta
clear Forecasts Port2
fprintf('25 Risk Adjusted BM and HML Beta Portfolios \n');
head = {'L HML';'2';'3';'4';'H HML';};
fmt = ' %G6.2 ';
row1 = { 'L BM'; '2';'3';'4';'H BM';'Average';};
Port2=zeros(6,5);
Port2(1:5,:)= BMHrsd_mean;
Port2(6,:)  = mean(BMHrsd_mean);
Forecasts =[Port2; ];
texprint(Forecasts, ' %12.3f ', head, row1);

% Conditional Beta 
clear Forecasts Port2
fprintf('25 Risk Adjusted BM and Conditional Beta Portfolios \n');
head = {'L Beta';'2';'3';'4';'H Beta';};
fmt = ' %G6.2 ';
row1 = { 'L BM'; '2';'3';'4';'H BM';'Average';};
Port2=zeros(6,5);
Port2(1:5,:)= BMCBTrsd_mean;
Port2(6,:)  = mean(BMCBTrsd_mean);
Forecasts =[Port2; ];
texprint(Forecasts, ' %12.3f ', head, row1);

% First Beta then BM
% HML Beta
clear Forecasts Port2
fprintf('25 First HML Beta and then BM Portfolios \n');
head = {'L HML';'2';'3';'4';'H HML';};
fmt = ' %G6.2 ';
row1 = { 'L BM'; '2';'3';'4';'H BM';'Average';};
Port2=zeros(6,5);
Port2(1:5,:)= HBM_mean';
Port2(6,:)  = mean(BMH_mean');
Forecasts =[Port2; ];
texprint(Forecasts, ' %12.3f ', head, row1);

% Conditional Beta 
clear Forecasts Port2
fprintf('25 First Conditional Beta and then BM Portfolios \n');
head = { 'L Beta';'2';'3';'4';'H Beta';};
fmt = ' %G6.2 ';
row1 = { 'L BM'; '2';'3';'4';'H BM';'Average';};
Port2=zeros(6,5);
Port2(1:5,:)= CBTBM_mean';
Port2(6,:)  = mean(CBTBM_mean');
Forecasts =[Port2; ];
texprint(Forecasts, ' %12.3f ', head, row1);

% unconditional CAPM
% 10 BM portfolios
fprintf('Unconditional CAPM: 10 BM \n')
head = {'Growth';'2';'3';'4';'5';'6';'7';'8';'9';'Value';'V-G'};
fmt = ' %G6.2 ';
row1 = {'mean';'std';'alpha';'t(alpha)';'2.5Con';'97.5Con';'beta';'t(beta)';'2.5Con';'97.5Con';'AdjR2';};
Forecasts=[BTM11_VW'; BTM11_VW_STD';alpha_CAPM';t_alpha_CAPM';alpha_CAPM_95Conf; beta_CAPM';t_beta_CAPM';beta_CAPM_95Conf;R2vadj_CAPM'; ];
texprint(Forecasts, ' %6.2f ', head, row1);
 